//
//  Network.swift
//  GitHub
//
//  Created by 曹云霄 on 2017/12/5.
//  Copyright © 2017年 曹云霄. All rights reserved.
//
import Moya
import SwiftyJSON


struct Network {
    
    // 请求头
    static let publicParamEndpointClosure = { (target: Service) -> Endpoint<Service> in
        let url = target.baseURL.absoluteString + target.path
        let endpoint = Endpoint<Service>(url: url, sampleResponseClosure: { .networkResponse(200, target.sampleData) }, method: target.method, task: target.task, httpHeaderFields: target.headers)
        if let userModel = AppManager.shareInstance.userModel {
            return endpoint.adding(newHTTPHeaderFields: ["enterprise" : userModel.enterpriseUuid,"Content-Type" : "application/json;charset=utf-8"])
        }
        return endpoint.adding(newHTTPHeaderFields: ["Content-Type" : "application/json;charset=utf-8"])
    }
    
    // 设置超时时间
    static let myRequestClosure = {(endpoint: Endpoint<Service>, closure: MoyaProvider<Service>.RequestResultClosure) -> Void in
        do {
           var urlRequest = try endpoint.urlRequest()
            urlRequest.timeoutInterval = 30
            closure(.success(urlRequest))
        } catch  {
            closure(.failure(MoyaError.requestMapping(endpoint.url)))
        }
    }

    // 监听请求状态
    static let myNetworkActivityPlugin = NetworkActivityPlugin { (type,target)  in
        switch type {
        case .began:
            ShowLoadingView(kWindow)
            UIApplication.shared.isNetworkActivityIndicatorVisible = true
        case .ended:
            HideLoadingView(kWindow)
            UIApplication.shared.isNetworkActivityIndicatorVisible = false
        }
    }

    // 公告请求对象
    static let provider = MoyaProvider<Service>(endpointClosure: publicParamEndpointClosure,requestClosure: myRequestClosure,plugins: [myNetworkActivityPlugin])

    // MARK: -取消所有请求
    static func cancelAllRequest() {
        provider.manager.session.invalidateAndCancel()
    }

    /// 公共请求方法
    ///
    /// - Parameters:
    ///   - target:  请求方法
    ///   - success: 请求成功
    ///   - error:   请求失败
    static func request(target: Service, success: @escaping (JSON) -> Void, failure: @escaping (MoyaError) -> Void) {
        provider.request(target) { result in
            switch result {
            /// 请求成功后先通过json解析数据，如果解析失败使用jsonString重试一次
            case let .success(response):
                //打印参数、返回
                print("url = " + (response.request?.url?.absoluteString)!)
                do {
                    if let data = response.request!.httpBody {
                        let body = try JSON(data: data)
                        print(body)
                    }
                    let result = try JSON(data: response.data)
                    print(result)
                }catch {}
                
                do {
                    guard response.statusCode == 200 else {
                        failure(MoyaError.statusCode(response))
                        return
                    }
                    let json = try response.mapJSON()
                    success(JSON(json))
                } catch {
                    switch (error) {
                    case MoyaError.jsonMapping(response):
                        do {
                            let jsonString = try response.mapString()
                            success(JSON(jsonString))
                        }catch {
                            failure(error as! MoyaError)
                        }
                    default:
                        failure(error as! MoyaError)
                        break
                    }
                }
                break
            case let .failure(error):
                failure(error)
                print(error.localizedDescription)
                break
            }
        }
    }
}
