//
//  RepairOrderAddTableViewController.swift
//  IFS
//
//  Created by 曹云霄 on 2018/1/3.
//  Copyright © 2018年 上海勾芒信息科技有限公司. All rights reserved.
//

import UIKit
import ImagePicker
import SKPhotoBrowser
import RxSwift
import RxCocoa



class RepairOrderAddTableViewController: BaseTableViewController {
    
    /// RxSwift自动释放
    let disposeBag = DisposeBag()
    /// 来源
    @IBOutlet weak var sourceLabel: UILabel!
    /// 服务类型
    @IBOutlet weak var serviceTypeLabel: UILabel!
    /// 优先级
    @IBOutlet weak var priorityLabel: UILabel!
    /// 报事人
    @IBOutlet weak var originatorLabel: UILabel!
    /// 接报时间
    @IBOutlet weak var originatorTimeLabel: UILabel!
    /// 指定时间
    @IBOutlet weak var specifiedTimeLabel: UILabel!
    /// 位置
    @IBOutlet weak var specifiedLocationLabel: UILabel!
    /// 设施信息
    @IBOutlet weak var facilitiesSwitch: UISwitch!
    /// 设施
    @IBOutlet weak var deviceTitleLabel: UILabel!
    /// 设施位置
    @IBOutlet weak var deviceLocationLabel: UITextField!
    /// 设置故障时间
    @IBOutlet weak var deviceFaultTimeLabel: UILabel!
    /// 联系电话
    @IBOutlet weak var phoneNumberText: UITextField!
    /// 工单描述
    @IBOutlet weak var describeTextView: IQTextView!
    /// 图片附件高度
    var attachmentHeight: CGFloat! = 0
    /// 默认section高度
    let sectionHeight: CGFloat! = 30
    
    /// 图片附件
    lazy final var attachmentVc: PhotoAttachmentViewController = {
       var attachmentVc = PhotoAttachmentViewController.instantiateViewController(.Function) as! PhotoAttachmentViewController
        attachmentVc.delegate = self
        return attachmentVc
    }()
    
    /// RepairOrderAddViewModel
    /// 新建工单ViewModel
    lazy final var repairOrderAddViewModel: RepairOrderAddViewModel = {
        var repairOrderAddViewModel = RepairOrderAddViewModel()
        return repairOrderAddViewModel
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        uiConfigAction()
        setupPhotoAttachmentVc()
        bindingRxAction()
    }
    
    // MARK: - UI
    fileprivate func uiConfigAction() {
        describeTextView.placeholder = "工单描述"
        tableView.register(UINib(nibName: RepairAttachmentTableViewCell.name(), bundle: nil), forCellReuseIdentifier: RepairAttachmentTableViewCell.name())
        describeTextView.textContainerInset = UIEdgeInsetsMake(0, 0, 0, 0)
        navigationItem.rightBarButtonItem = createButtonItem("提交", nil, self, #selector(RepairOrderAddTableViewController.submitButtonClickAction))
    }
    
    // MARK: - 保存图片附件至沙盒
    fileprivate func saveImage(currentImage: UIImage, persent: CGFloat, imageName: String) ->String {
        if let imageData = UIImageJPEGRepresentation(currentImage, persent) as NSData? {
            let fullPath = NSHomeDirectory().appending("/Documents/Files").appending(imageName)
            imageData.write(toFile: fullPath, atomically: true)
            return fullPath
        }
        return ""
    }
    
    // MARK: - 提交工单
    @objc fileprivate func submitButtonClickAction() {
        var fileUrlArray = Array<String>()
        for i in 0..<attachmentVc.attachmentViewModel.photoAttachments.count - 1 {
            let image = attachmentVc.attachmentViewModel.photoAttachments[i]
            let imageName = randomMD5()
            let fileUrl = saveImage(currentImage: image.attachment as! UIImage, persent: 0.8, imageName: imageName + ".jpg")
            fileUrlArray.append(fileUrl)
        }
        let entityUuid = randomMD5()
        for i in 0..<fileUrlArray.count {
            let fileUrl = fileUrlArray[i]
            let model = UploadPhotoAttachmentModel(randomMD5(), fileUrl, ATTACHMENT_TYPE.BILL_ATTACHMENT_TYPE.rawValue,entityUuid)
            Network.request(target: .UploadAttachment(model), success: {[weak self] (json) in
                let model = AttachmentReponseModel(fromJson: json)
                if model.success {
                    if i == fileUrlArray.count - kONE {
                        if self!.repairOrderAddViewModel.repairOrderAvailable(self!.facilitiesSwitch) {
                            self?.repairOrderAddViewModel.saveOrderModel.attachmentId = entityUuid
                            self?.repairOrderAddViewModel.submitRepairOrder().subscribe(onNext: {[weak self] (result) in
                                ShowMessage("提交成功")
                                self?.popVC()
                            }).disposed(by: self!.disposeBag)
                        }
                    }
                }
            }, failure: { (error) in
                ShowMessage(error.localizedDescription)
                HideLoadingView(kWindow)
                return
            })
        }
    }
    
    // MARK: - 生成随机字符串
    fileprivate func randomMD5() -> String {
        let identifier = CFUUIDCreate(nil)
        let identifierString = CFUUIDCreateString(nil, identifier) as String
        let cStr = identifierString.cString(using: .utf8)
        var digest = [UInt8](repeating: 0, count: Int(CC_MD5_DIGEST_LENGTH))
        CC_MD5(cStr, CC_LONG(strlen(cStr)), &digest)
        var output = String()
        for i in digest {
            output = output.appendingFormat("%02X", i)
        }
        return output;
    }
    
    // MARK: - 设置附件VC
    fileprivate func setupPhotoAttachmentVc(){
        addChildViewController(attachmentVc)
        attachmentVc.attachmentViewModel.photoAttachments.append(PhotoAttachmentModel(add_Photo, .ATTACHMENT_ADD))
        attachmentHeight = attachmentVc.attachmentHeight(kONE,nil)
    }
    
    // MARK: - 绑定RxSwift事件
    fileprivate func bindingRxAction() {
        facilitiesSwitch.rx.controlEvent(UIControlEvents.valueChanged).subscribe(onNext: {[weak self] (event) in
            self?.tableView.reloadData()
        }).disposed(by: disposeBag)
        
        describeTextView.rx.text.subscribe(onNext: {[weak self] (string) in
            self?.repairOrderAddViewModel.saveOrderModel.descriptionField = string
        }).disposed(by: disposeBag)
    }
    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.section == REPAIR_ORDER_SECTION.ATTACHMENT.rawValue {
            let attachmentCell = tableView.dequeueReusableCell(withIdentifier: RepairAttachmentTableViewCell.name(), for: indexPath) as! RepairAttachmentTableViewCell
            attachmentVc.view.frame = CGRect(x: 15, y: 35, width: kWidth - 30, height: attachmentHeight)
            attachmentCell.contentView.addSubview(attachmentVc.view)
            return attachmentCell
        }
        return super.tableView(tableView, cellForRowAt: indexPath)
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        if indexPath.section == REPAIR_ORDER_SECTION.ATTACHMENT.rawValue {
            return attachmentHeight + 50
        }
        return super.tableView(tableView, heightForRowAt: indexPath)
    }
    
    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if section == REPAIR_ORDER_SECTION.FACILITY.rawValue && !facilitiesSwitch.isOn {
            return kZERO
        }
        if section == REPAIR_ORDER_SECTION.ATTACHMENT.rawValue {
            return kONE
        }
        return super.tableView(tableView, numberOfRowsInSection: section)
    }
    
    override func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == REPAIR_ORDER_SECTION.BASIC.rawValue {
            return sectionHeight
        }
        if section == REPAIR_ORDER_SECTION.FACILITY.rawValue {
            if facilitiesSwitch.isOn {
                return sectionHeight
            }
            return kSectionZero
        }
        return kSectionTen
    }
    
    override func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        return kSectionZero
    }
    
    override func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        if section == REPAIR_ORDER_SECTION.FACILITY.rawValue && !facilitiesSwitch.isOn {
            return UIView()
        }
        return super.tableView(tableView, viewForHeaderInSection: section)
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        repairOrderAddViewModel.didSelectRowAtIndePath(indexPath, self)
    }
    
    deinit {
        print("释放")
    }
}

// MARK: - 更新附件高度
extension RepairOrderAddTableViewController: PhotoAttachmentDelegate {
    func updatePhotoAttachment(_ height: CGFloat, _ key: String?) {
        attachmentHeight = height
        tableView.reloadData()
    }
}












