//
//  RepairOrderEditSonOrderViewController.swift
//  IFS
//
//  Created by 曹云霄 on 2018/1/8.
//  Copyright © 2018年 上海勾芒信息科技有限公司. All rights reserved.
//

import UIKit
import RxSwift
import RxCocoa


class RepairOrderEditSonOrderViewController: BaseTableViewController {

    /// RxSwift自动释放
    let disposeBag = DisposeBag()
    /// 默认section高度
    let sectionHeight: CGFloat! = 30
    /// 维修前图片附件高度
    var beforAttachmentHeight: CGFloat! = 0
    /// 维修后图片附件高度
    var afterAttachmentHeight: CGFloat! = 0
    /// 工单数据
    open var billModel: BillDetailExecute!
    
    /// 主管指引、备件、工具
    @IBOutlet weak var toolOptionLabel: UILabel!
    /// 安全、防护、围封、环保
    @IBOutlet weak var protectOptionLabel: UILabel!
    /// 出工人
    @IBOutlet weak var workPersonLabel: UILabel!
    /// 到位时间
    @IBOutlet weak var placeTimeLabel: UILabel!
    /// 离开时间
    @IBOutlet weak var leaveTimeLabel: UILabel!
    /// 故障原因
    @IBOutlet weak var failureCauseLabel: UILabel!
    /// 处理方式
    @IBOutlet weak var processModeLabel: UILabel!
    /// 人工费
    @IBOutlet weak var costLaborText: UITextField!
    /// 物料费用
    @IBOutlet weak var materialCostLabel: UILabel!
    /// 描述
    @IBOutlet weak var describeTextView: IQTextView!
    
    /// 维修前图片附件
    lazy final var beforAttachmentVc: PhotoAttachmentViewController = {
        var attachmentVc = PhotoAttachmentViewController.instantiateViewController(.Function) as! PhotoAttachmentViewController
        attachmentVc.delegate = self
        return attachmentVc
    }()
    
    /// 维修后图片附件
    lazy final var afterAttachmentVc: PhotoAttachmentViewController = {
        var attachmentVc = PhotoAttachmentViewController.instantiateViewController(.Function) as! PhotoAttachmentViewController
        attachmentVc.delegate = self
        return attachmentVc
    }()
    
    /// 子工单详情ViewModel
    lazy final var childBillViewModel: ReparirChildBillViewModel = {
        var childBillViewModel = ReparirChildBillViewModel()
        return childBillViewModel
    }()
    
    /// 主工单详情ViewModel
    lazy final var orderDetailViewModel: RepairOrderDetailViewModel = {
        var orderDetailViewModel = RepairOrderDetailViewModel()
        return orderDetailViewModel
    }()
    
    override func viewDidLoad() {
        super.viewDidLoad()

        setupTableView()
        setupPhotoAttachmentVc()
        getBillDetailAction()
    }
    
    // MARK: - 获取子工单详情
    fileprivate func getBillDetailAction() {
        orderDetailViewModel.billDetailModel = BillDetailData(billModel.state)
        orderDetailViewModel.queryRepairOrderDetailAction(billModel.uuid).subscribe {[weak self] (event) in
            self?.tableView.reloadData()
            }.disposed(by: disposeBag)
    }
    
    // MARK: - 设置附件VC
    fileprivate func setupPhotoAttachmentVc(){
        addChildViewController(afterAttachmentVc)
        addChildViewController(beforAttachmentVc)
        beforAttachmentVc.attachmentViewModel.photoAttachments.append(PhotoAttachmentModel(add_Photo, .ATTACHMENT_ADD))
        afterAttachmentVc.attachmentViewModel.photoAttachments.append(PhotoAttachmentModel(add_Photo, .ATTACHMENT_ADD))
        afterAttachmentHeight = afterAttachmentVc.attachmentHeight(kONE,after)
        beforAttachmentHeight = beforAttachmentVc.attachmentHeight(kONE,befor)
    }
    
    // MARK: - TableView
    fileprivate func setupTableView() {
        tableView.register(RepairAfterAttachmentTableViewCell.classForCoder(), forCellReuseIdentifier: RepairAfterAttachmentTableViewCell.name())
        tableView.register(RepairBeforAttachmentTableViewCell.classForCoder(), forCellReuseIdentifier: RepairBeforAttachmentTableViewCell.name())
        let completeView = RepairOrderEditSonOrderCompleteView.instantiateFromNib() as! RepairOrderEditSonOrderCompleteView
        completeView.completeBtn.rx.controlEvent(UIControlEvents.touchUpInside).subscribe(onNext: {[weak self] (event) in
            self?.gotoChildBillDetailVc()
        }).disposed(by: disposeBag)
        tableView.tableFooterView = completeView
        navigationItem.rightBarButtonItem = createButtonItem("保存", nil, self, #selector(RepairOrderEditSonOrderViewController.saveButtonClickAction))
    }
    
    // MARK: - 提交、保存工单
    @objc fileprivate func saveButtonClickAction() {
        childBillViewModel.saveChildBillAction().subscribe(onNext: {[weak self] () in
            ShowMessage("保存成功")
            self?.popVC()
        }).disposed(by: disposeBag)
    }
    
    // MARK: - 跳转子工单详情
    fileprivate func gotoChildBillDetailVc() {
        self.performSegue(withIdentifier: RepairOrderChildBillDetailViewController.name(), sender: nil)
    }
    
    // MARK: - UITableViewDelegate,UITableViewDataSource
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if indexPath.section == CHILD_BILL_EDIT_SECTION.BEFORATTACHMENT.rawValue {
            let beforCell = tableView.dequeueReusableCell(withIdentifier: RepairBeforAttachmentTableViewCell.name(), for: indexPath)
            beforAttachmentVc.view.frame = CGRect(x: 15, y: 8, width: kWidth - 30, height: beforAttachmentHeight)
            beforCell.contentView.addSubview(beforAttachmentVc.view)
            return beforCell
        }
        if indexPath.section == CHILD_BILL_EDIT_SECTION.AFTERATTACHMENT.rawValue {
            let afterCell = tableView.dequeueReusableCell(withIdentifier: RepairAfterAttachmentTableViewCell.name(), for: indexPath)
            afterAttachmentVc.view.frame = CGRect(x: 15, y: 8, width: kWidth - 30, height: afterAttachmentHeight)
            afterCell.contentView.addSubview(afterAttachmentVc.view)
            return afterCell
        }
        return super.tableView(tableView, cellForRowAt: indexPath)
    }
    
    override func tableView(_ tableView: UITableView, heightForFooterInSection section: Int) -> CGFloat {
        return kSectionZero
    }
    
    override func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        if section == CHILD_BILL_EDIT_SECTION.DESCRIBE.rawValue {
            return 10
        }
        return sectionHeight
    }
    
    override func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        if indexPath.section == CHILD_BILL_EDIT_SECTION.BEFORATTACHMENT.rawValue {
            return beforAttachmentHeight + 20
        }
        if indexPath.section == CHILD_BILL_EDIT_SECTION.AFTERATTACHMENT.rawValue {
            return afterAttachmentHeight + 20
        }
        return super.tableView(tableView, heightForRowAt: indexPath)
    }
    
    override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        childBillViewModel.didSelectRowAtIndexPath(indexPath, self)
    }
}

extension RepairOrderEditSonOrderViewController: PhotoAttachmentDelegate {
    func updatePhotoAttachment(_ height: CGFloat, _ key: String?) {
        switch key! {
        case befor:
            beforAttachmentHeight = height
            break
        case after:
            afterAttachmentHeight = height
            break
        default:
            break
        }
        tableView.reloadData()
    }
}
